import { dataProperty, SwiftUICommon, BaseUIDataDriver } from './common';
import { Utils } from '@nativescript/core';
export * from './common';
const registry = new Map();
export function registerSwiftUI(id, callback) {
    registry.set(id, callback);
}
export class UIDataDriver extends BaseUIDataDriver {
    constructor(swiftUIProvider, owner) {
        super(owner);
        this.swiftUIProvider = swiftUIProvider;
    }
    createNativeView() {
        const vc = this.swiftUIProvider;
        return vc.view;
    }
    registerEvents(callback) {
        this.swiftUIProvider.onEvent = (data) => {
            callback(data);
        };
    }
    updateData(data) {
        this.swiftUIProvider.updateDataWithData(data);
    }
    onEvent(data) {
        const owner = this.owner.get();
        if (owner) {
            owner.notify({
                eventName: SwiftUICommon.swiftUIEventEvent,
                data: Utils.dataDeserialize(data),
            });
        }
    }
}
export class SwiftUI extends SwiftUICommon {
    createNativeView() {
        const generator = registry.get(this.swiftId);
        if (!generator) {
            console.warn('view not registered:', this.swiftId);
            return UIView.new();
        }
        this.driver = generator(this);
        return this.driver.createNativeView();
    }
    initNativeView() {
        if (!this.driver) {
            return;
        }
        this.driver.registerEvents((data) => {
            if (this.driver.onEvent) {
                this.driver.onEvent(data);
            }
            else {
                this.notify({
                    eventName: SwiftUI.swiftUIEventEvent,
                    data,
                });
            }
        });
    }
    disposeNativeView() {
        super.disposeNativeView();
        if (this.driver) {
            this.driver.destroyNativeView?.();
        }
    }
    [dataProperty.setNative](data) {
        this.updateData(data);
    }
    updateData(data) {
        this.driver?.updateData?.(data);
    }
}
export class WindowManager {
    static getWindow(id, isImmersive) {
        if (!WindowManager.currentWindows) {
            WindowManager.currentWindows = {};
        }
        if (!WindowManager.currentWindows[id]) {
            WindowManager.currentWindows[id] = new NativeScriptWindow(id, isImmersive);
        }
        return WindowManager.currentWindows[id];
    }
    static supportsMultipleScenes() {
        return UIApplication.sharedApplication.supportsMultipleScenes;
    }
}
export class NativeScriptWindow {
    constructor(id, isImmersive) {
        this.id = id;
        this.isImmersive = isImmersive;
    }
    open(props) {
        if (WindowManager.currentWindows[this.id]) {
            this._updateWindowData('NativeScriptWindowOpen', props);
        }
        return Promise.resolve();
    }
    close() {
        if (WindowManager.currentWindows[this.id]) {
            this._updateWindowData('NativeScriptWindowClose');
            NativeScriptWindowFactory.shared.removeWindowWithId(this.id);
            delete WindowManager.currentWindows[this.id];
        }
        return Promise.resolve();
    }
    update(props) {
        if (WindowManager.currentWindows[this.id]) {
            this._updateWindowData('NativeScriptWindowUpdate', props);
        }
        return Promise.resolve();
    }
    _updateWindowData(eventName, props) {
        if (props) {
            NativeScriptWindowFactory.shared.updateDataWithIdUpdates(this.id, props);
        }
        NSNotificationCenter.defaultCenter.postNotificationNameObjectUserInfo(eventName, null, Utils.dataSerialize({ type: this.id, isImmersive: this.isImmersive }));
    }
}
export class XR {
    static requestSession(sessionId, props) {
        if (!XR.currentSessionId) {
            XR.currentSessionId = sessionId;
            WindowManager.getWindow(sessionId, true).open(props);
        }
        return Promise.resolve();
    }
    static endSession() {
        if (XR.currentSessionId) {
            WindowManager.getWindow(XR.currentSessionId, true).close();
            XR.currentSessionId = null;
        }
        return Promise.resolve();
    }
}
//# sourceMappingURL=index.ios.js.map